@@ -1,6 +1,6 @@
 /* xfce4-mpc-plugin.c
  *
- * Copyright (c) 2006-2008 Landry Breuil (landry at fr.homeunix.org / gaston at gcu.info)
+ * Copyright (c) 2006-2009 Landry Breuil (landry at fr.homeunix.org / gaston at gcu.info)
  * This code is licenced under a BSD-style licence.
  * (OpenBSD variant modeled after the ISC licence)
  * All rights reserved.
@@ -23,13 +23,14 @@
 #endif
 
 #include <libxfcegui4/libxfcegui4.h>
+#include <exo/exo.h>
 #include <string.h>
+#include <stdlib.h>
 #include <glib/gprintf.h>
 
 #define DEFAULT_MPD_HOST "localhost"
 #define DEFAULT_MPD_PORT 6600
-#define DIALOG_ENTRY_WIDTH 15
-#define STRLENGTH 32
+#define DIALOG_ENTRY_WIDTH 20
 
 #include "xfce4-mpc-plugin.h"
 
@@ -38,11 +39,18 @@ mpc_free (XfcePanelPlugin * plugin, t_mpc * mpc)
 {
    DBG ("!");
 
-   g_object_unref (mpc->tips);
    mpd_free(mpc->mo);
    g_free (mpc);
 }
 
+static void
+button_set_sized_image(GtkWidget *button, gchar *icon, gint size)
+{
+   GtkWidget *image;
+   image = gtk_image_new_from_pixbuf(xfce_themed_icon_load(icon, size));
+   gtk_button_set_image(GTK_BUTTON(button), image);
+}
+
 static void
 mpc_set_orientation (XfcePanelPlugin * plugin, GtkOrientation orientation, t_mpc * mpc)
 {
@@ -55,23 +63,17 @@ static gboolean
 mpc_set_size (XfcePanelPlugin * plugin, int size, t_mpc * mpc)
 {
    DBG ("size=%d",size);
+   gtk_container_set_border_width (GTK_CONTAINER (mpc->frame), (size > 26 ? 2 : 0));
 
-   if (size > 26)
-   {
-      gtk_container_set_border_width (GTK_CONTAINER (mpc->frame), 2);
-      if (xfce_panel_plugin_get_orientation (plugin) == GTK_ORIENTATION_HORIZONTAL)
-         gtk_widget_set_size_request (GTK_WIDGET (mpc->frame), -1, size-2 );
-      else
-         gtk_widget_set_size_request (GTK_WIDGET (mpc->frame), size-4 , -1);
-   }
+   if (xfce_panel_plugin_get_orientation (plugin) == GTK_ORIENTATION_HORIZONTAL)
+     gtk_widget_set_size_request (GTK_WIDGET (mpc->frame), -1, size);
    else
-   {
-      gtk_container_set_border_width (GTK_CONTAINER (mpc->frame), 0);
-      if (xfce_panel_plugin_get_orientation (plugin) == GTK_ORIENTATION_HORIZONTAL)
-         gtk_widget_set_size_request (GTK_WIDGET (mpc->frame), -1, size);
-      else
-         gtk_widget_set_size_request (GTK_WIDGET (mpc->frame), size, -1);
-   }
+     gtk_widget_set_size_request (GTK_WIDGET (mpc->frame), size, -1);
+
+   button_set_sized_image(mpc->prev, "gtk-media-previous-ltr", size - 3);
+   button_set_sized_image(mpc->next, "gtk-media-next-ltr", size - 3);
+   button_set_sized_image(mpc->toggle, "gtk-media-pause", size - 3);
+   button_set_sized_image(mpc->stop, "gtk-media-stop", size - 3);
    return TRUE;
 }
 
@@ -100,16 +102,26 @@ mpc_read_config (XfcePanelPlugin * plugin, t_mpc * mpc)
       g_free (mpc->mpd_host);
    if (mpc->mpd_password != NULL)
       g_free (mpc->mpd_password);
+   if (mpc->tooltip_format != NULL)
+      g_free (mpc->tooltip_format);
+   if (mpc->playlist_format != NULL)
+      g_free (mpc->playlist_format);
+   if (mpc->client_appl != NULL)
+      g_free (mpc->client_appl);
 
    mpc->mpd_host = g_strdup(xfce_rc_read_entry (rc, "mpd_host",  DEFAULT_MPD_HOST));
    mpc->mpd_port = xfce_rc_read_int_entry (rc, "mpd_port", DEFAULT_MPD_PORT);
    mpc->mpd_password = g_strdup(xfce_rc_read_entry (rc, "mpd_password", ""));
    mpc->show_frame = xfce_rc_read_bool_entry (rc, "show_frame", TRUE);
-   mpc->client_appl = g_strdup(xfce_rc_read_entry (rc, "client_appl",  ""));
+   mpc->client_appl = g_strdup(xfce_rc_read_entry (rc, "client_appl", "SETME"));
+   mpc->tooltip_format = g_strdup(xfce_rc_read_entry (rc, "tooltip_format", "Volume : %vol%% - Mpd %status%%newline%%%artist% - %album% -/- (#%track%) %title%"));
+   mpc->playlist_format = g_strdup(xfce_rc_read_entry (rc, "playlist_format", "%artist% - %album% -/- (#%track%) %title%"));
+
    label = gtk_bin_get_child(GTK_BIN(mpc->appl));
-   g_sprintf(str, "%s %s", _("Launch"), mpc->client_appl);
+   g_snprintf(str, sizeof(str), "%s %s", _("Launch"), mpc->client_appl);
    gtk_label_set_text(GTK_LABEL(label),str);
-   DBG ("Settings : %s@%s:%d\nframe:%d\nappl:%s", mpc->mpd_password, mpc->mpd_host, mpc->mpd_port, mpc->show_frame, mpc->client_appl);
+
+   DBG ("Settings read : %s@%s:%d\nframe:%d\nappl:%s\ntooltip:%s\nplaylist:%s", mpc->mpd_password, mpc->mpd_host, mpc->mpd_port, mpc->show_frame, mpc->client_appl, mpc->tooltip_format, mpc->playlist_format);
    xfce_rc_close (rc);
 }
 
@@ -141,6 +153,8 @@ static void mpc_write_config (XfcePanelPlugin * plugin, t_mpc * mpc)
    xfce_rc_write_entry (rc, "mpd_password", mpc->mpd_password);
    xfce_rc_write_bool_entry (rc, "show_frame", mpc->show_frame);
    xfce_rc_write_entry (rc, "client_appl", mpc->client_appl);
+   xfce_rc_write_entry (rc, "tooltip_format", mpc->tooltip_format);
+   xfce_rc_write_entry (rc, "playlist_format", mpc->playlist_format);
 
    xfce_rc_close (rc);
 }
@@ -162,18 +176,28 @@ mpc_dialog_apply_options (t_mpc_dialog *dialog)
 {
    DBG ("!");
    GtkWidget *label;
-   char str[30];
+   char str[128];
 
    t_mpc *mpc = dialog->mpc;
-   mpc->mpd_host = g_strndup(gtk_entry_get_text(GTK_ENTRY(dialog->textbox_host)),STRLENGTH);
+   mpc->mpd_host = g_strdup(gtk_entry_get_text(GTK_ENTRY(dialog->textbox_host)));
    mpc->mpd_port = atoi(gtk_entry_get_text(GTK_ENTRY(dialog->textbox_port)));
-   mpc->mpd_password = g_strndup(gtk_entry_get_text(GTK_ENTRY(dialog->textbox_password)),STRLENGTH);
-   mpc->client_appl = g_strndup(gtk_entry_get_text(GTK_ENTRY(dialog->textbox_client_appl)),STRLENGTH);
+   mpc->mpd_password = g_strdup(gtk_entry_get_text(GTK_ENTRY(dialog->textbox_password)));
+   mpc->client_appl = g_strdup(gtk_entry_get_text(GTK_ENTRY(dialog->textbox_client_appl)));
+   mpc->tooltip_format = g_strdup(gtk_entry_get_text(GTK_ENTRY(dialog->textbox_tooltip_format)));
+   mpc->playlist_format = g_strdup(gtk_entry_get_text(GTK_ENTRY(dialog->textbox_playlist_format)));
+
+   if (0 == strlen(mpc->client_appl))
+      mpc->client_appl = g_strdup("SETME");
+   if (0 == strlen(mpc->tooltip_format))
+      mpc->tooltip_format = g_strdup("Volume : %vol%% - Mpd %status%%newline%%artist% - %album% -/- (#%track%) %title%");
+   if (0 == strlen(mpc->playlist_format))
+      mpc->playlist_format = g_strdup("%artist% - %album% -/- (#%track%) %title%");
+
    label = gtk_bin_get_child(GTK_BIN(mpc->appl));
-   g_sprintf(str, "%s %s", _("Launch"), mpc->client_appl);
+   g_snprintf(str, sizeof(str), "%s %s", _("Launch"), mpc->client_appl);
    gtk_label_set_text(GTK_LABEL(label),str);
 
-   DBG ("Apply: host=%s, port=%d, passwd=%s, appl=%s", mpc->mpd_host, mpc->mpd_port, mpc->mpd_password, mpc->client_appl);
+   DBG ("Apply: host=%s, port=%d, passwd=%s, appl=%s\ntooltip=%s\nplaylist=%s", mpc->mpd_host, mpc->mpd_port, mpc->mpd_password, mpc->client_appl, mpc->tooltip_format, mpc->playlist_format);
 
    mpd_disconnect(mpc->mo);
    mpd_set_hostname(mpc->mo,mpc->mpd_host);
@@ -243,11 +267,13 @@ mpc_create_options (XfcePanelPlugin * plugin, t_mpc* mpc)
    gtk_widget_show (vbox);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), vbox, TRUE, TRUE, 0);
 
-   table = gtk_table_new(4,2,FALSE);
+   table = gtk_table_new(6,2,FALSE);
    gtk_table_attach_defaults(GTK_TABLE(table),gtk_label_new(_("Host : ")),0,1,0,1);
    gtk_table_attach_defaults(GTK_TABLE(table),gtk_label_new(_("Port : ")),0,1,1,2);
    gtk_table_attach_defaults(GTK_TABLE(table),gtk_label_new(_("Password : ")),0,1,2,3);
-   gtk_table_attach_defaults(GTK_TABLE(table),gtk_label_new(_("Client : ")),0,1,3,4);
+   gtk_table_attach_defaults(GTK_TABLE(table),gtk_label_new(_("MPD Client : ")),0,1,3,4);
+   gtk_table_attach_defaults(GTK_TABLE(table),gtk_label_new(_("Tooltip Format : ")),0,1,4,5);
+   gtk_table_attach_defaults(GTK_TABLE(table),gtk_label_new(_("Playlist Format : ")),0,1,5,6);
 
    dialog->textbox_host = gtk_entry_new();
    gtk_entry_set_width_chars(GTK_ENTRY(dialog->textbox_host),DIALOG_ENTRY_WIDTH);
@@ -271,6 +297,21 @@ mpc_create_options (XfcePanelPlugin * plugin, t_mpc* mpc)
    gtk_entry_set_text(GTK_ENTRY(dialog->textbox_client_appl),mpc->client_appl);
    gtk_table_attach_defaults(GTK_TABLE(table),dialog->textbox_client_appl,1,2,3,4);
 
+   dialog->textbox_tooltip_format = gtk_entry_new();
+   gtk_entry_set_width_chars(GTK_ENTRY(dialog->textbox_tooltip_format),DIALOG_ENTRY_WIDTH);
+   gtk_entry_set_text(GTK_ENTRY(dialog->textbox_tooltip_format),mpc->tooltip_format);
+   gtk_table_attach_defaults(GTK_TABLE(table),dialog->textbox_tooltip_format,1,2,4,5);
+
+   dialog->textbox_playlist_format = gtk_entry_new();
+   gtk_entry_set_width_chars(GTK_ENTRY(dialog->textbox_playlist_format),DIALOG_ENTRY_WIDTH);
+   gtk_entry_set_text(GTK_ENTRY(dialog->textbox_playlist_format),mpc->playlist_format);
+   gtk_table_attach_defaults(GTK_TABLE(table),dialog->textbox_playlist_format,1,2,5,6);
+
+   gtk_widget_set_tooltip_text (dialog->textbox_host, _("Hostname or IP address"));
+   gtk_widget_set_tooltip_text (dialog->textbox_client_appl, _("Graphical MPD Client to launch in plugin context menu"));
+   gtk_widget_set_tooltip_text (dialog->textbox_playlist_format, _("Variables : %artist%, %album%, %track% and %title%"));
+   gtk_widget_set_tooltip_text (dialog->textbox_tooltip_format, _("Variables : %vol%, %status%, %newline%, %artist%, %album%, %track% and %title%"));
+
    gtk_widget_show_all (table);
    gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
 
@@ -306,64 +347,80 @@ mpc_repeat_toggled(GtkWidget *widget, t_mpc* mpc)
    mpd_player_set_repeat(mpc->mo, gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget)));
 }
 
+
 void
-format_song_display(mpd_Song* song, gchar* str)
+str_replace(GString *str, gchar* pattern, gchar* replacement)
 {
-   /* buf may contain stuff, care to append text */
-   if (!song->artist || !song->title)
-      g_sprintf(str,"%s%s", str, song->file);
-   else if (!song->album)
-      g_sprintf(str,"%s%s - %s", str, song->artist, song->title);
-   else if (!song->track)
-      g_sprintf(str,"%s%s - %s -/- %s", str, song->artist, song->album, song->title);
-   else
-      g_sprintf(str,"%s%s - %s -/- (#%s) %s", str, song->artist, song->album, song->track, song->title);
+   if (!replacement)
+      return;
+   gchar *nstr = exo_str_replace(str->str, pattern, replacement);
+   g_string_assign(str, nstr);
+   g_free(nstr);
+}
+
+void
+format_song_display(mpd_Song* song, GString *str, t_mpc* mpc)
+{
+   if (0 == str->len)
+      g_string_assign(str, mpc->playlist_format);
+
+   /* replace %artist% by song->artist, etc */
+   str_replace(str, "%artist%", song->artist);
+   str_replace(str, "%album%", song->album);
+   str_replace(str, "%title%", song->title);
+   str_replace(str, "%track%", song->track);
 }
 
 static void
 enter_cb(GtkWidget *widget, GdkEventCrossing* event, t_mpc* mpc)
 {
    mpd_Song *song;
-   gchar str[512];
-
+   GString *str = NULL;
+   gchar vol[20];
    DBG("!");
 
    if (mpd_status_update(mpc->mo) != MPD_OK)
    {
       if (!mpc_plugin_reconnect (mpc) || mpd_status_update (mpc->mo) != MPD_OK)
       {
-         gtk_tooltips_set_tip (mpc->tips, widget, _(".... not connected ?"), NULL);
+         gtk_widget_set_tooltip_text (widget, _(".... not connected ?"));
          return;
       }
    }
+   str = g_string_new(mpc->tooltip_format);
 
-   g_sprintf(str, "Volume : %d%%", mpd_status_get_volume(mpc->mo));
-
+   /* replace %vol% by mpd_status_get_volume(mpc->mo) */
+   g_sprintf(vol, "%d", mpd_status_get_volume(mpc->mo));
+   str_replace(str, "%vol%", vol);
+   /* newlines */
+   str_replace(str, "%newline%", "\n");
+   /* replace %status% by mpd status string */
    switch (mpd_player_get_state(mpc->mo))
    {
       case MPD_PLAYER_PLAY:
-         g_sprintf(str, "%s - Mpd Playing\n",str);
+         str_replace(str, "%status%", "Playing");
          break;
       case MPD_PLAYER_PAUSE:
-         g_sprintf(str, "%s - Mpd Paused\n",str);
+         str_replace(str, "%status%", "Paused");
          break;
       case MPD_PLAYER_STOP:
-         g_sprintf(str, "%s - Mpd Stopped\n",str);
+         str_replace(str, "%status%", "Stopped");
          break;
       default:
-         g_sprintf(str, "%s - Mpd state ?\n",str);
+         str_replace(str, "%status%", "state ?");
          break;
    }
    song = mpd_playlist_get_current_song(mpc->mo);
    if (song && song->id != -1)
-      format_song_display(song, str);
+      format_song_display(song, str, mpc);
    else
-      g_sprintf(str,"%sFailed to get song info ?", str);
+      g_string_assign(str, "Failed to get song info ?");
 
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mpc->random), mpd_player_get_random(mpc->mo));
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mpc->repeat), mpd_player_get_repeat(mpc->mo));
 
-   gtk_tooltips_set_tip (mpc->tips, widget, str, NULL);
+   gtk_widget_set_tooltip_text (widget, str->str);
+   g_string_free(str, TRUE);
 }
 
 static void
@@ -392,10 +449,11 @@ show_playlist (t_mpc* mpc)
    GtkTreeIter iter;
    GtkTreePath *path_to_cur;
    GtkCellRenderer *renderer;
-   gchar str[512];
+   GString *str;
    int current;
    MpdData *mpd_data;
 
+   str = g_string_new('\0');
    if (mpc->playlist)
    {
       gtk_window_present(GTK_WINDOW(mpc->playlist));
@@ -439,23 +497,24 @@ show_playlist (t_mpc* mpc)
       DBG ("Got playlist, filling treeview");
       do
       {
-         str[0]='\0';
-         format_song_display(mpd_data->song, str);
+         g_string_erase(str, 0, -1);
+         format_song_display(mpd_data->song, str, mpc);
 
          gtk_list_store_append (liststore, &iter);
          if (current == mpd_data->song->pos)
          {
-            gtk_list_store_set (liststore, &iter, 0, "gtk-media-play", 1, str, 2, mpd_data->song->pos, 3, mpd_data->song->id, -1);
+            gtk_list_store_set (liststore, &iter, 0, "gtk-media-play", 1, str->str, 2, mpd_data->song->pos, 3, mpd_data->song->id, -1);
             path_to_cur = gtk_tree_model_get_path(GTK_TREE_MODEL(liststore), &iter);
             gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(treeview), path_to_cur, NULL, TRUE, 0.5, 0);
             gtk_tree_view_set_cursor(GTK_TREE_VIEW(treeview), path_to_cur, NULL, FALSE);
             gtk_tree_path_free(path_to_cur);
          }
          else
-            gtk_list_store_set (liststore, &iter, 0, "gtk-cdrom", 1, str, 2, mpd_data->song->pos, 3, mpd_data->song->id, -1);
+            gtk_list_store_set (liststore, &iter, 0, "gtk-cdrom", 1, str->str, 2, mpd_data->song->pos, 3, mpd_data->song->id, -1);
       } while (NULL != (mpd_data = mpd_data_get_next (mpd_data)));
       gtk_widget_show_all(mpc->playlist);
    }
+   g_string_free(str, TRUE);
 }
 
 static void
@@ -554,7 +613,7 @@ scroll_cb(GtkWidget *widget, GdkEventScroll* event, t_mpc* mpc)
    {
       if (!mpc_plugin_reconnect (mpc) || mpd_status_update (mpc->mo) != MPD_OK)
       {
-         gtk_tooltips_set_tip (mpc->tips, widget, _(".... not connected ?"), NULL);
+         gtk_widget_set_tooltip_text(widget, _(".... not connected ?"));
          return;
       }
    }
@@ -566,19 +625,14 @@ scroll_cb(GtkWidget *widget, GdkEventScroll* event, t_mpc* mpc)
    mpd_status_set_volume(mpc->mo,curvol);
 }
 
-static void
-new_button_with_img(XfcePanelPlugin * plugin, GtkWidget *parent, GtkWidget *button, gchar *icon, gpointer cb, gpointer data)
+static GtkWidget*
+new_button_with_cbk(XfcePanelPlugin * plugin, GtkWidget *parent, gpointer cb, gpointer data)
 {
-   GtkWidget *image;
-
-   button = (GtkWidget*) xfce_create_panel_button();
-   image = gtk_image_new_from_stock(icon,GTK_ICON_SIZE_BUTTON); /* FIXME for small panels */
-
-   gtk_button_set_image(GTK_BUTTON(button),image);
+   GtkWidget *button = xfce_create_panel_button();
    xfce_panel_plugin_add_action_widget (plugin, button);
-   gtk_widget_show (GTK_WIDGET(button));
    g_signal_connect (G_OBJECT(button), "button_press_event", G_CALLBACK(cb), data);
-   gtk_box_pack_start (GTK_BOX(parent), button, TRUE,TRUE,0);
+   gtk_box_pack_start (GTK_BOX(parent), button, TRUE, TRUE, 0);
+   return button;
 }
 
 static t_mpc*
@@ -591,9 +645,6 @@ mpc_create (XfcePanelPlugin * plugin)
    mpc = g_new0 (t_mpc, 1);
 
    mpc->plugin = plugin;
-   mpc->tips = gtk_tooltips_new ();
-   g_object_ref (mpc->tips);
-   gtk_object_sink (GTK_OBJECT (mpc->tips));
 
    mpc->frame = gtk_frame_new (NULL);
    gtk_frame_set_shadow_type (GTK_FRAME (mpc->frame), GTK_SHADOW_IN);
@@ -604,15 +655,15 @@ mpc_create (XfcePanelPlugin * plugin)
    g_signal_connect (G_OBJECT(mpc->ebox), "scroll_event", G_CALLBACK(scroll_cb), mpc);
    gtk_widget_show (mpc->ebox);
 
-   mpc->box = (GtkWidget*) xfce_hvbox_new(xfce_panel_plugin_get_orientation(plugin), FALSE, 0);
+   mpc->box = xfce_hvbox_new(xfce_panel_plugin_get_orientation(plugin), FALSE, 0);
 
    gtk_container_add (GTK_CONTAINER(mpc->ebox), mpc->box);
    gtk_container_add (GTK_CONTAINER(mpc->frame), mpc->ebox);
 
-   new_button_with_img(plugin, mpc->box, mpc->prev, GTK_STOCK_MEDIA_PREVIOUS, G_CALLBACK(prev), mpc);
-   new_button_with_img(plugin, mpc->box, mpc->stop, GTK_STOCK_MEDIA_STOP, G_CALLBACK(stop), mpc);
-   new_button_with_img(plugin, mpc->box, mpc->toggle, GTK_STOCK_MEDIA_PAUSE, G_CALLBACK(toggle), mpc);
-   new_button_with_img(plugin, mpc->box, mpc->next, GTK_STOCK_MEDIA_NEXT, G_CALLBACK(next), mpc);
+   mpc->prev = new_button_with_cbk(plugin, mpc->box, G_CALLBACK(prev), mpc);
+   mpc->stop = new_button_with_cbk(plugin, mpc->box, G_CALLBACK(stop), mpc);
+   mpc->toggle = new_button_with_cbk(plugin, mpc->box, G_CALLBACK(toggle), mpc);
+   mpc->next = new_button_with_cbk(plugin, mpc->box, G_CALLBACK(next), mpc);
 
    mpc->random = gtk_check_menu_item_new_with_label (_("Random"));
    g_signal_connect (G_OBJECT(mpc->random), "toggled", G_CALLBACK (mpc_random_toggled), mpc);
@@ -676,10 +727,13 @@ mpc_construct (XfcePanelPlugin * plugin)
 
    /* create widgets */
    mpc = mpc_create (plugin);
+   /* default values when no configuration is found */
    mpc->mpd_host = g_strdup(DEFAULT_MPD_HOST);
    mpc->mpd_port = DEFAULT_MPD_PORT;
    mpc->mpd_password = g_strdup("");
-   mpc->client_appl = g_strdup("");
+   mpc->client_appl = g_strdup("SETME");
+   mpc->tooltip_format = g_strdup("Volume : %vol%% - Mpd %status%%newline%%artist% - %album% -/- (#%track%) %title%");
+   mpc->playlist_format = g_strdup("%artist% - %album% -/- (#%track%) %title%");
    mpc->show_frame = TRUE;
    mpc->playlist = NULL;
 
