@@ -1,6 +1,6 @@
 /* xfce4-mpc-plugin.h
  *
- * Copyright (c) 2006-2008 Landry Breuil (landry at fr.homeunix.org / gaston at gcu.info)
+ * Copyright (c) 2006-2009 Landry Breuil (landry at fr.homeunix.org / gaston at gcu.info)
  * This code is licenced under a BSD-style licence.
  * (OpenBSD variant modeled after the ISC licence)
  * All rights reserved.
@@ -33,7 +33,6 @@
 
 typedef struct {
    XfcePanelPlugin *plugin;
-   GtkTooltips *tips;
    GtkWidget *frame,*ebox,*box,*prev,*stop,*toggle,*next,*random,*repeat,*appl,*about;
    GtkWidget *playlist;
    gboolean show_frame;
@@ -43,6 +42,8 @@ typedef struct {
    gchar* mpd_host;
    gint mpd_port;
    gchar * mpd_password;
+   gchar * tooltip_format;
+   gchar * playlist_format;
    gboolean mpd_repeat;
    gboolean mpd_random;
 } t_mpc;
@@ -53,6 +54,8 @@ typedef struct {
    GtkWidget *textbox_port;
    GtkWidget *textbox_password;
    GtkWidget *textbox_client_appl;
+   GtkWidget *textbox_tooltip_format;
+   GtkWidget *textbox_playlist_format;
    GtkWidget *checkbox_frame;
 } t_mpc_dialog;
 
